@@ -91,6 +91,12 @@ struct ETreePriv {
 
 	int reflow_idle_id;
 
+	int table_model_change_id;
+	int table_row_change_id;
+	int table_cell_change_id;
+	int table_rows_inserted_id;
+	int table_rows_deleted_id;
+
 	GnomeCanvas *header_canvas, *table_canvas;
 
 	GnomeCanvasItem *header_item, *root;
@@ -103,6 +109,7 @@ struct ETreePriv {
 	/*
 	 * Configuration settings
 	 */
+	guint alternating_row_colors : 1;
 	guint horizontal_draw_grid : 1;
 	guint vertical_draw_grid : 1;
 	guint draw_focus : 1;
@@ -175,6 +182,32 @@ static gint e_tree_drag_source_event_cb (GtkWidget      *widget,
 
 static gint et_focus (GtkContainer *container, GtkDirectionType direction);
 
+static void
+et_disconnect_from_etta (ETree *et)
+{
+	if (et->priv->table_model_change_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->priv->etta),
+				       et->priv->table_model_change_id);
+	if (et->priv->table_row_change_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->priv->etta),
+				       et->priv->table_row_change_id);
+	if (et->priv->table_cell_change_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->priv->etta),
+				       et->priv->table_cell_change_id);
+	if (et->priv->table_rows_inserted_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->priv->etta),
+				       et->priv->table_rows_inserted_id);
+	if (et->priv->table_rows_deleted_id != 0)
+		gtk_signal_disconnect (GTK_OBJECT (et->priv->etta),
+				       et->priv->table_rows_deleted_id);
+
+	et->priv->table_model_change_id = 0;
+	et->priv->table_row_change_id = 0;
+	et->priv->table_cell_change_id = 0;
+	et->priv->table_rows_inserted_id = 0;
+	et->priv->table_rows_deleted_id = 0;
+}
+
 static void
 et_destroy (GtkObject *object)
 {
@@ -184,6 +217,9 @@ et_destroy (GtkObject *object)
 		g_source_remove(et->priv->reflow_idle_id);
 	et->priv->reflow_idle_id = 0;
 
+	et_disconnect_from_etta (et);
+
+	gtk_object_unref (GTK_OBJECT (et->priv->etta));
 	gtk_object_unref (GTK_OBJECT (et->priv->model));
 	gtk_object_unref (GTK_OBJECT (et->priv->sorted));
 	gtk_object_unref (GTK_OBJECT (et->priv->full_header));
@@ -193,9 +229,15 @@ et_destroy (GtkObject *object)
 	if (et->priv->spec)
 		gtk_object_unref (GTK_OBJECT (et->priv->spec));
 
-	if (et->priv->header_canvas != NULL)
+	if (et->priv->sorter)
+		gtk_object_unref (GTK_OBJECT (et->priv->sorter));
+
+	if (et->priv->header_canvas)
 		gtk_widget_destroy (GTK_WIDGET (et->priv->header_canvas));
 
+	if (et->priv->site)
+		e_tree_drag_source_unset (et);
+
 	gtk_widget_destroy (GTK_WIDGET (et->priv->table_canvas));
 
 	g_free(et->priv);
@@ -211,57 +253,58 @@ e_tree_init (GtkObject *object)
 
 	GTK_WIDGET_SET_FLAGS (e_tree, GTK_CAN_FOCUS);
 
-	gtk_table->homogeneous = FALSE;
+	gtk_table->homogeneous                           = FALSE;
 
-	e_tree->priv = g_new(ETreePriv, 1);
+	e_tree->priv                                     = g_new(ETreePriv, 1);
 
-	e_tree->priv->model = NULL;
-	e_tree->priv->sorted = NULL;
-	e_tree->priv->etta = NULL;
+	e_tree->priv->model                              = NULL;
+	e_tree->priv->sorted                             = NULL;
+	e_tree->priv->etta                               = NULL;
 
-	e_tree->priv->full_header = NULL;
-	e_tree->priv->header = NULL;
+	e_tree->priv->full_header                        = NULL;
+	e_tree->priv->header                             = NULL;
 
-	e_tree->priv->sort_info = NULL;
-	e_tree->priv->sorter = NULL;
-	e_tree->priv->reflow_idle_id = 0;
+	e_tree->priv->sort_info                          = NULL;
+	e_tree->priv->sorter                             = NULL;
+	e_tree->priv->reflow_idle_id                     = 0;
 
-	e_tree->priv->horizontal_draw_grid = 1;
-	e_tree->priv->vertical_draw_grid = 1;
-	e_tree->priv->draw_focus = 1;
-	e_tree->priv->cursor_mode = E_CURSOR_SIMPLE;
-	e_tree->priv->length_threshold = 200;
+	e_tree->priv->alternating_row_colors		 = 1;
+	e_tree->priv->horizontal_draw_grid               = 1;
+	e_tree->priv->vertical_draw_grid                 = 1;
+	e_tree->priv->draw_focus                         = 1;
+	e_tree->priv->cursor_mode                        = E_CURSOR_SIMPLE;
+	e_tree->priv->length_threshold                   = 200;
 
-	e_tree->priv->row_selection_active = FALSE;
-	e_tree->priv->horizontal_scrolling = FALSE;
+	e_tree->priv->row_selection_active               = FALSE;
+	e_tree->priv->horizontal_scrolling               = FALSE;
 
-	e_tree->priv->drop_row = -1;
-	e_tree->priv->drop_path = NULL;
-	e_tree->priv->drop_col = -1;
+	e_tree->priv->drop_row                           = -1;
+	e_tree->priv->drop_path                          = NULL;
+	e_tree->priv->drop_col                           = -1;
 
-	e_tree->priv->drag_row = -1;
-	e_tree->priv->drag_path = NULL;
-	e_tree->priv->drag_col = -1;
+	e_tree->priv->drag_row                           = -1;
+	e_tree->priv->drag_path                          = NULL;
+	e_tree->priv->drag_col                           = -1;
 
-	e_tree->priv->site = NULL;
-	e_tree->priv->drag_source_button_press_event_id = 0;
+	e_tree->priv->site                               = NULL;
+	e_tree->priv->drag_source_button_press_event_id  = 0;
 	e_tree->priv->drag_source_motion_notify_event_id = 0;
 
 #ifdef E_TREE_USE_TREE_SELECTION
-	e_tree->priv->selection = E_SELECTION_MODEL(e_tree_selection_model_new());
+	e_tree->priv->selection                          = E_SELECTION_MODEL(e_tree_selection_model_new());
 #else
-	e_tree->priv->selection = E_SELECTION_MODEL(e_table_selection_model_new());
+	e_tree->priv->selection                          = E_SELECTION_MODEL(e_table_selection_model_new());
 #endif
-	e_tree->priv->spec = NULL;
+	e_tree->priv->spec                               = NULL;
 
-	e_tree->priv->header_canvas = NULL;
-	e_tree->priv->table_canvas = NULL;
+	e_tree->priv->header_canvas                      = NULL;
+	e_tree->priv->table_canvas                       = NULL;
 
-	e_tree->priv->header_item = NULL;
-	e_tree->priv->root = NULL;
+	e_tree->priv->header_item                        = NULL;
+	e_tree->priv->root                               = NULL;
 
-	e_tree->priv->white_item = NULL;
-	e_tree->priv->item = NULL;
+	e_tree->priv->white_item                         = NULL;
+	e_tree->priv->item                               = NULL;
 }
 
 /* Grab_focus handler for the ETree */
@@ -485,57 +528,47 @@ item_key_press (ETableItem *eti, int row, int col, GdkEvent *event, ETree *et)
 {
 	int return_val = 0;
 	GdkEventKey *key = (GdkEventKey *) event;
-	GdkEventButton click;
 	ETreePath path;
+	int y, row_local, col_local;
+	GtkAdjustment *vadj;
 
 	switch (key->keyval) {
 	case GDK_Page_Down:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas)),
-			CLAMP(gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->value +
-			      (gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size - 20),
-			      0,
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->upper -
-			      gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->priv->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size - 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->priv->table_canvas),
-				 (GdkEvent *) &click);
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas));
+		y = CLAMP(vadj->value + (2 * vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_tree_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_tree_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->priv->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->priv->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	case GDK_Page_Up:
-		gtk_adjustment_set_value(
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas)),
-			gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->value -
-			(gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas))->page_size - 20));
-		click.type = GDK_BUTTON_PRESS;
-		click.window = GTK_LAYOUT (et->priv->table_canvas)->bin_window;
-		click.send_event = key->send_event;
-		click.time = key->time;
-		click.x = 30;
-		click.y = 1;
-		click.state = key->state;
-		click.button = 1;
-		gtk_widget_event(GTK_WIDGET(et->priv->table_canvas),
-				 (GdkEvent *) &click);
+		vadj = gtk_layout_get_vadjustment (GTK_LAYOUT (et->priv->table_canvas));
+		y = CLAMP(vadj->value - (vadj->page_size - 50), 0, vadj->upper);
+		y -= vadj->value;
+		e_tree_get_cell_at (et, 30, y, &row_local, &col_local);
+		row_local = e_tree_view_to_model_row (et, row_local);
+		col_local = e_selection_model_cursor_col (E_SELECTION_MODEL (et->priv->selection));
+		e_selection_model_select_as_key_press (E_SELECTION_MODEL (et->priv->selection), row_local, col_local, key->state);
 		return_val = 1;
 		break;
 	case '=':
 	case GDK_Right:
-		path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
-		e_tree_table_adapter_node_set_expanded (et->priv->etta, path, TRUE);
+		if (row != -1) {
+			path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
+			if (path)
+				e_tree_table_adapter_node_set_expanded (et->priv->etta, path, TRUE);
+		}
 		return_val = 1;
 		break;
 	case '-':
 	case GDK_Left:
-		path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
-		e_tree_table_adapter_node_set_expanded (et->priv->etta, path, FALSE);
+		if (row != -1) {
+			path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
+			if (path)
+				e_tree_table_adapter_node_set_expanded (et->priv->etta, path, FALSE);
+		}
 		return_val = 1;
 		break;
 	default:
@@ -564,6 +597,7 @@ et_build_item (ETree *et)
 					 "ETableHeader", et->priv->header,
 					 "ETableModel", et->priv->etta,
 					 "selection_model", et->priv->selection,
+					 "alternating_row_colors", et->priv->alternating_row_colors,
 					 "horizontal_draw_grid", et->priv->horizontal_draw_grid,
 					 "vertical_draw_grid", et->priv->vertical_draw_grid,
 					 "drawfocus", et->priv->draw_focus,
@@ -676,8 +710,6 @@ e_tree_set_state_object(ETree *e_tree, ETableState *state)
 	if (e_tree->priv->header)
 		gtk_object_unref(GTK_OBJECT(e_tree->priv->header));
 	e_tree->priv->header = e_table_state_to_header (GTK_WIDGET(e_tree), e_tree->priv->full_header, state);
-	if (e_tree->priv->header)
-		gtk_object_ref(GTK_OBJECT(e_tree->priv->header));
 
 	gtk_object_set (GTK_OBJECT (e_tree->priv->header),
 			"width", (double) (GTK_WIDGET(e_tree->priv->table_canvas)->allocation.width),
@@ -820,7 +852,7 @@ gchar          *e_tree_get_state                 (ETree               *e_tree)
  */
 void
 e_tree_save_state (ETree      *e_tree,
-		    const gchar *filename)
+		   const gchar *filename)
 {
 	ETableState *state;
 
@@ -829,6 +861,57 @@ e_tree_save_state (ETree      *e_tree,
 	gtk_object_unref(GTK_OBJECT(state));
 }
 
+static void
+et_table_model_changed (ETableModel *model, ETree *et)
+{
+	if (et->priv->horizontal_scrolling)
+		e_table_header_update_horizontal(et->priv->header);
+}
+
+static void
+et_table_row_changed (ETableModel *table_model, int row, ETree *et)
+{
+	et_table_model_changed (table_model, et);
+}
+
+static void
+et_table_cell_changed (ETableModel *table_model, int view_col, int row, ETree *et)
+{
+	et_table_model_changed (table_model, et);
+}
+
+static void
+et_table_rows_inserted (ETableModel *table_model, int row, int count, ETree *et)
+{
+	et_table_model_changed (table_model, et);
+}
+
+static void
+et_table_rows_deleted (ETableModel *table_model, int row, int count, ETree *et)
+{
+	et_table_model_changed (table_model, et);
+}
+
+static void
+et_connect_to_etta (ETree *et)
+{
+	et->priv->table_model_change_id = gtk_signal_connect (GTK_OBJECT (et->priv->etta), "model_changed",
+							      GTK_SIGNAL_FUNC (et_table_model_changed), et);
+
+	et->priv->table_row_change_id = gtk_signal_connect (GTK_OBJECT (et->priv->etta), "model_row_changed",
+							    GTK_SIGNAL_FUNC (et_table_row_changed), et);
+
+	et->priv->table_cell_change_id = gtk_signal_connect (GTK_OBJECT (et->priv->etta), "model_cell_changed",
+							     GTK_SIGNAL_FUNC (et_table_cell_changed), et);
+
+	et->priv->table_rows_inserted_id = gtk_signal_connect (GTK_OBJECT (et->priv->etta), "model_rows_inserted",
+							       GTK_SIGNAL_FUNC (et_table_rows_inserted), et);
+
+	et->priv->table_rows_deleted_id = gtk_signal_connect (GTK_OBJECT (et->priv->etta), "model_rows_deleted",
+							      GTK_SIGNAL_FUNC (et_table_rows_deleted), et);
+
+}
+
 static ETree *
 et_real_construct (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 		   ETableSpecification *specification, ETableState *state)
@@ -840,6 +923,7 @@ et_real_construct (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 	else
 		ete = e_table_extras_new();
 
+	e_tree->priv->alternating_row_colors = specification->alternating_row_colors;
 	e_tree->priv->horizontal_draw_grid = specification->horizontal_draw_grid;
 	e_tree->priv->vertical_draw_grid = specification->vertical_draw_grid;
 	e_tree->priv->draw_focus = specification->draw_focus;
@@ -863,6 +947,8 @@ et_real_construct (ETree *e_tree, ETreeModel *etm, ETableExtras *ete,
 
 	e_tree->priv->etta = E_TREE_TABLE_ADAPTER(e_tree_table_adapter_new(E_TREE_MODEL(e_tree->priv->sorted)));
 
+	et_connect_to_etta (e_tree);
+
 	gtk_widget_push_visual (gdk_rgb_get_visual ());
 	gtk_widget_push_colormap (gdk_rgb_get_cmap ());
 
@@ -1001,7 +1087,7 @@ e_tree_construct_from_spec_file (ETree *e_tree, ETreeModel *etm, ETableExtras *e
 GtkWidget *
 e_tree_new (ETreeModel *etm, ETableExtras *ete, const char *spec, const char *state)
 {
-	ETree *e_tree;
+	ETree *e_tree, *ret_val;
 
 	g_return_val_if_fail(etm != NULL, NULL);
 	g_return_val_if_fail(E_IS_TREE_MODEL(etm), NULL);
@@ -1010,15 +1096,19 @@ e_tree_new (ETreeModel *etm, ETableExtras *ete, const char *spec, const char *st
 
 	e_tree = gtk_type_new (e_tree_get_type ());
 
-	e_tree = e_tree_construct (e_tree, etm, ete, spec, state);
+	ret_val = e_tree_construct (e_tree, etm, ete, spec, state);
 
-	return GTK_WIDGET (e_tree);
+	if (ret_val == NULL) {
+		gtk_object_unref (GTK_OBJECT (e_tree));
+	}
+
+	return (GtkWidget *) ret_val;
 }
 
 GtkWidget *
 e_tree_new_from_spec_file (ETreeModel *etm, ETableExtras *ete, const char *spec_fn, const char *state_fn)
 {
-	ETree *e_tree;
+	ETree *e_tree, *ret_val;
 
 	g_return_val_if_fail(etm != NULL, NULL);
 	g_return_val_if_fail(E_IS_TREE_MODEL(etm), NULL);
@@ -1027,9 +1117,13 @@ e_tree_new_from_spec_file (ETreeModel *etm, ETableExtras *ete, const char *spec_
 
 	e_tree = gtk_type_new (e_tree_get_type ());
 
-	e_tree = e_tree_construct_from_spec_file (e_tree, etm, ete, spec_fn, state_fn);
+	ret_val = e_tree_construct_from_spec_file (e_tree, etm, ete, spec_fn, state_fn);
 
-	return GTK_WIDGET (e_tree);
+	if (ret_val == NULL) {
+		gtk_object_unref (GTK_OBJECT (e_tree));
+	}
+
+	return (GtkWidget *) ret_val;
 }
 
 void
@@ -1070,6 +1164,8 @@ e_tree_get_cursor (ETree *e_tree)
 	gtk_object_get(GTK_OBJECT(e_tree->priv->selection),
 		       "cursor_row", &row,
 		       NULL);
+	if (row == -1)
+		return NULL;
 	path = e_tree_table_adapter_node_at_row(E_TREE_TABLE_ADAPTER(e_tree->priv->etta), row);
 	path = e_tree_sorted_view_to_model_path(e_tree->priv->sorted, path);
 	return path;
@@ -1103,34 +1199,6 @@ e_tree_selected_path_foreach     (ETree *e_tree,
 }
 #endif
 
-gint
-e_tree_selected_count     (ETree *e_tree)
-{
-	g_return_val_if_fail(e_tree != NULL, -1);
-	g_return_val_if_fail(E_IS_TREE(e_tree), -1);
-
-	return e_selection_model_selected_count(E_SELECTION_MODEL (e_tree->priv->selection));
-}
-
-void
-e_tree_select_all (ETree *tree)
-{
-	g_return_if_fail (tree != NULL);
-	g_return_if_fail (E_IS_TREE (tree));
-
-	e_selection_model_select_all (E_SELECTION_MODEL (tree->priv->selection));
-}
-
-void
-e_tree_invert_selection (ETree *tree)
-{
-	g_return_if_fail (tree != NULL);
-	g_return_if_fail (E_IS_TREE (tree));
-
-	e_selection_model_invert_selection (E_SELECTION_MODEL (tree->priv->selection));
-}
-
-
 EPrintable *
 e_tree_get_printable (ETree *e_tree)
 {
@@ -1397,6 +1465,41 @@ e_tree_get_tooltip (ETree *et)
 	return E_CANVAS(et->priv->table_canvas)->tooltip_window;
 }
 
+/**
+ * e_tree_get_model:
+ * @et: the ETree
+ *
+ * Returns the model upon which this ETree is based.
+ *
+ * Returns: the model
+ **/
+ETreeModel *
+e_tree_get_model (ETree *et)
+{
+	g_return_val_if_fail (et != NULL, NULL);
+	g_return_val_if_fail (E_IS_TREE (et), NULL);
+
+	return et->priv->model;
+}
+
+/**
+ * e_tree_get_selection_model:
+ * @et: the ETree
+ *
+ * Returns the selection model of this ETree.
+ *
+ * Returns: the selection model
+ **/
+ESelectionModel *
+e_tree_get_selection_model (ETree *et)
+{
+	g_return_val_if_fail (et != NULL, NULL);
+	g_return_val_if_fail (E_IS_TREE (et), NULL);
+
+	return et->priv->selection;
+}
+
+
 struct _ETreeDragSourceSite
 {
 	GdkModifierType    start_button_mask;
@@ -1689,6 +1792,40 @@ e_tree_get_cell_at (ETree *tree,
 	e_table_item_compute_location(E_TABLE_ITEM(tree->priv->item), &x, &y, row_return, col_return);
 }
 
+/**
+ * e_tree_get_cell_geometry:
+ * @tree: The tree.
+ * @row: The row to get the geometry of.
+ * @col: The col to get the geometry of.
+ * @x_return: Returns the x coordinate of the upper right hand corner of the cell with respect to the widget.
+ * @y_return: Returns the y coordinate of the upper right hand corner of the cell with respect to the widget.
+ * @width_return: Returns the width of the cell.
+ * @height_return: Returns the height of the cell.
+ * 
+ * Computes the data about this cell.
+ **/
+void
+e_tree_get_cell_geometry (ETree *tree,
+			  int row, int col,
+			  int *x_return, int *y_return,
+			  int *width_return, int *height_return)
+{
+	g_return_if_fail (tree != NULL);
+	g_return_if_fail (E_IS_TREE (tree));
+	g_return_if_fail (row >= 0);
+	g_return_if_fail (col >= 0);
+
+	/* FIXME it would be nice if it could handle a NULL row_return or
+	 * col_return gracefully.  */
+
+	e_table_item_get_cell_geometry(E_TABLE_ITEM(tree->priv->item), &row, &col, x_return, y_return, width_return, height_return);
+
+	if (x_return)
+		(*x_return) -= GTK_LAYOUT(tree->priv->table_canvas)->hadjustment->value;
+	if (y_return)
+		(*y_return) -= GTK_LAYOUT(tree->priv->table_canvas)->vadjustment->value;
+}
+
 static void
 et_drag_begin (GtkWidget *widget,
 	       GdkDragContext *context,
@@ -1780,8 +1917,7 @@ et_drag_motion(GtkWidget *widget,
 			    y,
 			    &row,
 			    &col);
-	if (et->priv->drop_row >= 0 && et->priv->drop_col >= 0 &&
-	    row != et->priv->drop_row && col != et->priv->drop_row) {
+	if (row != et->priv->drop_row && col != et->priv->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [TREE_DRAG_LEAVE],
 				 et->priv->drop_row,
@@ -1797,17 +1933,16 @@ et_drag_motion(GtkWidget *widget,
 	et->priv->drop_row = row;
 	et->priv->drop_path = path;
 	et->priv->drop_col = col;
-	if (row >= 0 && col >= 0)
-		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TREE_DRAG_MOTION],
-				 et->priv->drop_row,
-				 et->priv->drop_path,
-				 et->priv->drop_col,
-				 context,
-				 x,
-				 y,
-				 time,
-				 &ret_val);
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TREE_DRAG_MOTION],
+			 et->priv->drop_row,
+			 et->priv->drop_path,
+			 et->priv->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
 	return ret_val;
 }
 
@@ -1830,8 +1965,7 @@ et_drag_drop(GtkWidget *widget,
 	path = e_tree_table_adapter_node_at_row(et->priv->etta, row);
 	path = e_tree_sorted_view_to_model_path(et->priv->sorted, path);
 
-	if (et->priv->drop_row >= 0 && et->priv->drop_col >= 0 &&
-	    row != et->priv->drop_row && col != et->priv->drop_row) {
+	if (row != et->priv->drop_row && col != et->priv->drop_row) {
 		gtk_signal_emit (GTK_OBJECT (et),
 				 et_signals [TREE_DRAG_LEAVE],
 				 et->priv->drop_row,
@@ -1839,32 +1973,30 @@ et_drag_drop(GtkWidget *widget,
 				 et->priv->drop_col,
 				 context,
 				 time);
-		if (row >= 0 && col >= 0)
-			gtk_signal_emit (GTK_OBJECT (et),
-					 et_signals [TREE_DRAG_MOTION],
-					 row,
-					 path,
-					 col,
-					 context,
-					 x,
-					 y,
-					 time,
-					 &ret_val);
-	}
-	et->priv->drop_row = row;
-	et->priv->drop_path = path;
-	et->priv->drop_col = col;
-	if (row >= 0 && col >= 0)
 		gtk_signal_emit (GTK_OBJECT (et),
-				 et_signals [TREE_DRAG_DROP],
-				 et->priv->drop_row,
-				 et->priv->drop_path,
-				 et->priv->drop_col,
+				 et_signals [TREE_DRAG_MOTION],
+				 row,
+				 path,
+				 col,
 				 context,
 				 x,
 				 y,
 				 time,
 				 &ret_val);
+	}
+	et->priv->drop_row = row;
+	et->priv->drop_path = path;
+	et->priv->drop_col = col;
+	gtk_signal_emit (GTK_OBJECT (et),
+			 et_signals [TREE_DRAG_DROP],
+			 et->priv->drop_row,
+			 et->priv->drop_path,
+			 et->priv->drop_col,
+			 context,
+			 x,
+			 y,
+			 time,
+			 &ret_val);
 	et->priv->drop_row = -1;
 	et->priv->drop_path = NULL;
 	et->priv->drop_col = -1;
