@@ -13,7 +13,9 @@
 #include <gal/e-table/e-cell-tree.h>
 #include <gal/e-table/e-cell-checkbox.h>
 #include <gal/e-table/e-table.h>
-#include <gal/e-table/e-tree-simple.h>
+#include <gal/e-table/e-tree-memory.h>
+#include <gal/e-table/e-tree-memory-callbacks.h>
+#include <gal/e-table/e-tree-table-adapter.h>
 
 #define COLS 4
 
@@ -49,61 +51,69 @@
 	        <column source=\"2\"/> \
 	        <column source=\"3\"/> \
 	        <grouping></grouping> \
-        </ETableState>
+        </ETableState> \
 </ETableSpecification>"
 
 GtkWidget *e_table;
+ETreeTableAdapter *adapter;
 
 /*
  * ETreeSimple callbacks
  * These are the callbacks that define the behavior of our custom model.
  */
 
+static GdkPixbuf *
+my_icon_at (ETreeModel *etm, ETreePath path, void *model_data)
+{
+	/* No icon, since the cell tree renderer takes care of the +/- icons itself. */
+	return NULL;
+}
+
 /* This function returns the number of columns in our ETableModel. */
 static int
-my_col_count (ETableModel *etc, void *data)
+my_column_count (ETreeModel *etc, void *data)
 {
 	return COLS;
 }
 
 /* This function duplicates the value passed to it. */
 static void *
-my_duplicate_value (ETableModel *etc, int col, const void *value, void *data)
+my_duplicate_value (ETreeModel *etc, int col, const void *value, void *data)
 {
 	return g_strdup (value);
 }
 
 /* This function frees the value passed to it. */
 static void
-my_free_value (ETableModel *etc, int col, void *value, void *data)
+my_free_value (ETreeModel *etc, int col, void *value, void *data)
 {
 	g_free (value);
 }
 
 /* This function creates an empty value. */
 static void *
-my_initialize_value (ETableModel *etc, int col, void *data)
+my_initialize_value (ETreeModel *etc, int col, void *data)
 {
 	return g_strdup ("");
 }
 
 /* This function reports if a value is empty. */
 static gboolean
-my_value_is_empty (ETableModel *etc, int col, const void *value, void *data)
+my_value_is_empty (ETreeModel *etc, int col, const void *value, void *data)
 {
 	return !(value && *(char *)value);
 }
 
 /* This function reports if a value is empty. */
 static char *
-my_value_to_string (ETableModel *etc, int col, const void *value, void *data)
+my_value_to_string (ETreeModel *etc, int col, const void *value, void *data)
 {
 	return g_strdup(value);
 }
 
 /* This function returns the value at a particular point in our ETreeModel. */
 static void *
-my_value_at (ETreeModel *etm, ETreePath *path, int col, void *model_data)
+my_value_at (ETreeModel *etm, ETreePath path, int col, void *model_data)
 {
 	if (e_tree_model_node_is_root (etm, path)) {
 		if (col == 0)
@@ -114,7 +124,7 @@ my_value_at (ETreeModel *etm, ETreePath *path, int col, void *model_data)
 	}
 	else {
 		switch (col) {
-		case 0: return e_tree_model_node_get_data (etm, path);
+		case 0: return e_tree_memory_node_get_data (E_TREE_MEMORY(etm), path);
 		case 1: return "Chris Toshok";
 		case 2: return "toshok@helixcode.com";
 		case 3: return "Jun 07 2000";
@@ -123,30 +133,23 @@ my_value_at (ETreeModel *etm, ETreePath *path, int col, void *model_data)
 	}
 }
 
-static GdkPixbuf *
-my_icon_at (ETreeModel *etm, ETreePath *path, void *model_data)
-{
-	/* No icon, since the cell tree renderer takes care of the +/- icons itself. */
-	return NULL;
-}
-
 /* This function sets the value at a particular point in our ETreeModel. */
 static void
-my_set_value_at (ETreeModel *etm, ETreePath *path, int col, const void *val, void *model_data)
+my_set_value_at (ETreeModel *etm, ETreePath path, int col, const void *val, void *model_data)
 {
 	if (e_tree_model_node_is_root (etm, path))
 		return;
 
 	if (col == 0) {
-		char *str = e_tree_model_node_get_data (etm, path);
+		char *str = e_tree_memory_node_get_data (E_TREE_MEMORY(etm), path);
 		g_free (str);
-		e_tree_model_node_set_data (etm, path, g_strdup(val));
+		e_tree_memory_node_set_data (E_TREE_MEMORY(etm), path, g_strdup(val));
 	}
 }
 
 /* This function returns whether a particular cell is editable. */
 static gboolean
-my_is_editable (ETreeModel *etm, ETreePath *path, int col, void *model_data)
+my_is_editable (ETreeModel *etm, ETreePath path, int col, void *model_data)
 {
 	if (col == 0)
 		return TRUE;
@@ -154,38 +157,46 @@ my_is_editable (ETreeModel *etm, ETreePath *path, int col, void *model_data)
 		return FALSE;
 }
 
+#if 0
 static gint
 ascending_compare (ETreeModel *model,
-		   ETreePath  *node1,
-		   ETreePath  *node2)
+		   ETreePath  node1,
+		   ETreePath  node2)
 {
 	char *path1, *path2;
-	path1 = e_tree_model_node_get_data (model, node1);
-	path2 = e_tree_model_node_get_data (model, node2);
+	ETreeMemory *etmm = E_TREE_MEMORY(model);
+
+	path1 = e_tree_memory_node_get_data (etmm, node1);
+	path2 = e_tree_memory_node_get_data (etmm, node2);
 
 	return g_strcasecmp (path1, path2);
 }
 
 static gint
 descending_compare (ETreeModel *model,
-		    ETreePath  *node1,
-		    ETreePath  *node2)
+		    ETreePath  node1,
+		    ETreePath  node2)
 {
 	return - ascending_compare (model, node1, node2);
 }
+#endif
 
 static void
 sort_descending (GtkButton *button, gpointer data)
 {
+#if 0
 	ETreeModel *model = E_TREE_MODEL (data);
 	e_tree_model_node_set_compare_function (model, e_tree_model_get_root (model), ascending_compare);
+#endif
 }
 
 static void
 sort_ascending (GtkButton *button, gpointer data)
 {
+#if 0
 	ETreeModel *model = E_TREE_MODEL (data);
 	e_tree_model_node_set_compare_function (model, e_tree_model_get_root (model), descending_compare);
+#endif
 }
 
 /* We create a window containing our new tree. */
@@ -195,41 +206,55 @@ create_tree (void)
 	GtkWidget *window, *frame, *button, *vbox;
 	int i, j;
 	ETreeModel *e_tree_model = NULL;
-	ETreePath *root_node;
+	ETreePath root_node;
+	ETreeMemory *etmm;
 
 	/* here we create our model.  This uses the functions we defined
 	   earlier. */
-	e_tree_model = e_tree_simple_new (my_col_count,
-					  my_duplicate_value,
-					  my_free_value,
-					  my_initialize_value,
-					  my_value_is_empty,
-					  my_value_to_string,
-					  my_icon_at,
-					  my_value_at,
-					  my_set_value_at,
-					  my_is_editable,
-					  NULL);
+	e_tree_model = e_tree_memory_callbacks_new (my_icon_at,
 
-	e_tree_model_load_expanded_state (e_tree_model, "expanded_state");
+						    my_column_count,
+
+						    NULL,
+						    NULL,
+
+						    NULL,
+						    NULL,
+
+						    my_value_at,
+						    my_set_value_at,
+						    my_is_editable,
+
+						    my_duplicate_value,
+						    my_free_value,
+						    my_initialize_value,
+						    my_value_is_empty,
+						    my_value_to_string,
+						    NULL);
+
+	etmm = E_TREE_MEMORY(e_tree_model);
+
+	adapter = E_TREE_TABLE_ADAPTER(e_tree_table_adapter_new(e_tree_model));
+
+	e_tree_table_adapter_load_expanded_state (adapter, "expanded_state");
 
 	/* create a root node with 5 children */
-	root_node = e_tree_model_node_insert (e_tree_model, NULL,
+	root_node = e_tree_memory_node_insert (etmm, NULL,
 					      0,
 					      NULL);
 
-	e_tree_model_root_node_set_visible (e_tree_model, TRUE);
+	e_tree_table_adapter_root_node_set_visible (adapter, TRUE);
 
 	for (i = 0; i < 5; i++){
 		char *id = g_strdup_printf ("First level child %d", i);
-		ETreePath *n = e_tree_model_node_insert (e_tree_model,
+		ETreePath n = e_tree_memory_node_insert (etmm,
 							 root_node, 0,
 							 id);
 		for (j = 0; j < 5; j ++) {
 			char *id = g_strdup_printf ("Second level child %d.%d", i, j);
-			e_tree_model_node_insert (e_tree_model,
-						  n, 0,
-						  id);
+			e_tree_memory_node_insert (etmm,
+						   n, 0,
+						   id);
 		}
 	}
 
@@ -249,12 +274,12 @@ create_tree (void)
 	 * the table we've created, the header, the model, and the
 	 * initial layout.  It does the rest.
 	 */
-	e_table = e_table_new (E_TABLE_MODEL(e_tree_model), NULL, INITIAL_SPEC, NULL);
+	e_table = e_table_new (E_TABLE_MODEL(adapter), NULL, INITIAL_SPEC, NULL);
 
 	if (!e_table) printf ("BAH!");
 
 	gtk_object_set (GTK_OBJECT (e_table),
-			"cursor_mode", E_TABLE_CURSOR_LINE,
+			"cursor_mode", E_CURSOR_LINE,
 			NULL);
 
 	/* Build the gtk widget hierarchy. */
